@@ -7,7 +7,7 @@ use pyo3::{prelude::*, types::PyTuple};
 use crate::{
     stream::{
         model::{DefaultContiguousCategoricalEntropyModel, DefaultLeakyQuantizer},
-        queue::RangeCoderState,
+        queue::{DecoderFrontendError, RangeCoderState},
         Decode, Encode,
     },
     Pos, Seek, UnwrapInfallible,
@@ -654,7 +654,6 @@ impl RangeDecoder {
     ///    stds  = np.array([ 5.2, 24.2,  3.1], dtype=np.float64)
     ///
     ///    # Decode a message from some example compressed data:
-    ///    # (needs `len(symbols) == len(means) == len(stds)`)
     ///    compressed = np.array([2655472005], dtype=np.uint32)
     ///    decoder = constriction.stream.queue.RangeDecoder(compressed)
     ///    symbols = decoder.decode(model_family, means, stds)
@@ -709,11 +708,14 @@ impl RangeDecoder {
                 if std > 0.0 && std.is_finite() && mean.is_finite() {
                     Ok(quantizer.quantize(Gaussian::new(mean, std)))
                 } else {
-                    Err(())
+                    Err(                pyo3::exceptions::PyValueError::new_err(
+                        "Invalid model parameters (`std` must be strictly positive and both `std` and `mean` must be finite.).",
+                    )
+    )
                 }
             }))
             .collect::<std::result::Result<Vec<_>, _>>()
-            .expect("We use constant `PRECISION`.");
+            ?;
 
         Ok(PyArray1::from_vec(py, symbols))
     }
@@ -784,8 +786,8 @@ impl RangeDecoder {
         Ok(PyArray1::from_iter(
             py,
             self.inner.decode_iid_symbols(amt, &model).map(|symbol| {
-                (symbol.expect("We use constant `PRECISION`.") as i32)
-                    .wrapping_add(min_supported_symbol)
+                let symbol = symbol.unwrap_or_else(|e| panic!("{}", e)) as i32;
+                symbol.wrapping_add(min_supported_symbol)
             }),
         ))
     }
@@ -861,7 +863,6 @@ impl RangeDecoder {
     /// stds  = np.array([ 5.2, 24.2,  3.1], dtype=np.float64)
     ///
     /// # Decode a message from some example compressed data:
-    /// # (needs `len(symbols) == len(means) == len(stds)`)
     /// compressed = np.array([2655472005], dtype=np.uint32)
     /// decoder = constriction.stream.queue.RangeDecoder(compressed)
     /// symbols = decoder.decode(model_family, means, stds)
@@ -898,10 +899,7 @@ impl RangeDecoder {
             0 => {
                 let mut symbol = 0;
                 model.0.as_parameterized(py, &mut |model| {
-                    symbol = self
-                        .inner
-                        .decode_symbol(EncoderDecoderModel(model))
-                        .expect("We use constant `PRECISION`.");
+                    symbol = self.inner.decode_symbol(EncoderDecoderModel(model))?;
                     Ok(())
                 })?;
                 return Ok(symbol.to_object(py));
@@ -914,8 +912,7 @@ impl RangeDecoder {
                             .inner
                             .decode_iid_symbols(amt, EncoderDecoderModel(model))
                         {
-                            let symbol = symbol.expect("We use constant `PRECISION`.");
-                            symbols.push(symbol);
+                            symbols.push(symbol?);
                         }
                         Ok(())
                     })?;
@@ -927,10 +924,7 @@ impl RangeDecoder {
 
         let mut symbols = Vec::with_capacity(model.0.len(&params[0])?);
         model.0.parameterize(py, params, false, &mut |model| {
-            let symbol = self
-                .inner
-                .decode_symbol(EncoderDecoderModel(model))
-                .expect("We use constant `PRECISION`.");
+            let symbol = self.inner.decode_symbol(EncoderDecoderModel(model))?;
             symbols.push(symbol);
             Ok(())
         })?;
@@ -999,3 +993,13 @@ impl RangeDecoder {
         Self { inner }
     }
 }
+
+impl From<DecoderFrontendError> for pyo3::PyErr {
+    fn from(err: DecoderFrontendError) -> Self {
+        match err {
+            DecoderFrontendError::InvalidData => {
+                pyo3::exceptions::PyAssertionError::new_err(err.to_string())
+            }
+        }
+    }
+}
